<?php

namespace Drupal\yandex_oauth;

use Drupal\Core\Database\Connection;

/**
 * Provides YandexOAuthTokens, a 'yandex_oauth' service class.
 */
class YandexOAuthTokens {

  /**
   * The database service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Static cache of DB select results.
   *
   * @var object[]
   */
  protected static $cache = [];

  /**
   * Constructs a new YandexOAuthTokens.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database service.
   */
  public function __construct(Connection $database) {
    $this->database = $database;
  }

  /**
   * Saves the access token information record to the database.
   *
   * @param string $name
   *   Yandex account name.
   * @param int $uid
   *   Site's user ID.
   * @param string $token
   *   The access token string.
   * @param int $expire
   *   Token expiration timestamp.
   *
   * @return int|null
   *   If the record insert or update failed, returns NULL. If it succeeded,
   *   returns one of Merge class constants, depending on the operation
   *   performed.
   */
  public function save($name, $uid, $token, $expire) {
    $fields = ['uid' => $uid, 'token' => $token, 'expire' => $expire];
    return $this->database->merge('yandex_oauth')
      ->fields($fields)
      ->key('name', $name)
      ->execute();
  }

  /**
   * Given an account name loads the access token information from a database.
   *
   * @param string $name
   *   Yandex account name.
   * @param bool $fresh
   *   (optional) Set to FALSE to load the record even if token has expired.
   *
   * @return object|false
   *   The object with token info if conditions matching token exists. Otherwise
   *   returns NULL. Fields of returning object:
   *   - name: The same account name given in an argument.
   *   - uid: ID of user associated with this Yandex account.
   *   - token: The access token string.
   *   - expire: Token expiration timestamp.
   */
  public function get($name, $fresh = TRUE) {
    if (!isset(self::$cache[$name])) {
      self::$cache[$name] = $this->database->select('yandex_oauth', 'yo')
        ->fields('yo')
        ->condition('name', $name)
        ->execute()
        ->fetchObject();
    }

    if (self::$cache[$name] && (self::$cache[$name]->expire > REQUEST_TIME || !$fresh)) {
      return self::$cache[$name];
    }
  }

}
